SystemOrganization addCategory: #'FuelFameExtension-Tests'!
SystemOrganization addCategory: #'FuelFameExtension-Core'!
SystemOrganization addCategory: #'FuelFameExtension-Core-Mappers'!
SystemOrganization addCategory: #'FuelFameExtension-Core-Clusters'!

FLProgressUpdaterAnalyzer subclass: #FLFameAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core'!

!FLFameAnalyzer commentStamp: '<historical>' prior: 0!
I am an analyzer with a specific graph traversing behavior. 

Given an object described in a Fame repository, I work with its Famix metamodel description, for know which references traverse.!

!FLFameAnalyzer classSide methodsFor: 'protected' stamp: 'MartinDias 8/18/2011 17:55'!
lightMappers

	^super lightMappers asOrderedCollection 
		addFirst: FLFameMapper;
		yourself
! !

!FLFameAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 8/19/2011 01:56'!
initialize

	super initialize.
	clusterization := FLFameClusterization new.! !

!FLSerializer class methodsFor: '*FuelFameExtension' stamp: 'MarianoMartinezPeck 5/18/2011 00:16'!
on: aStream serialize: entities describedWith: aMetamodel
	"Serializes the Fame entities on aStream"

	(self on: aStream)
		serialize: entities
		describedWith: aMetamodel.
! !

!FLSerializer methodsFor: '*FuelFameExtension' stamp: 'MartinDias 8/24/2011 20:16'!
serialize: entities describedWith: aMetamodel
	"Serializes the Fame entities on the stream"

	FLFameObjectCluster defaultRepository: 
		((FMRepository with: aMetamodel) 
			addAll: entities;
			yourself).
	self
		analyzer: FLFameAnalyzer newDefault;
		serialize: entities		
		analysisCheckpoints: entities.
			! !

TestCase subclass: #FLFameAnalyzerTest
	instanceVariableNames: 'analyzer repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Tests'!

!FLFameAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/24/2010 01:53'!
assert: aClass notDependsOn: yetNotVisitedClasses

	((repository metamodel descriptionOf: aClass) allComplexAttributes
		reject: [ :anAttribute | anAttribute isDerived ])
			do: [ :anAttribute | 
				self deny: (yetNotVisitedClasses anySatisfy: [ :aYetNotVisitedClass | 
					aYetNotVisitedClass isKindOf: anAttribute type implementingClass ]) ]! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/22/2010 17:50'!
repositoryWithSample
	
	^ (FMRepository with: MooseModel meta) 
			addAll: self sampleMooseModel entities;
			yourself.! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 2/23/2011 00:52'!
sampleMooseModel

	^ MooseSqueakClassPackageImporterTask new
		model: MooseModel new;
		addFromPackageNamed: 'Merlin';
		run;
		model.! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 8/12/2011 00:08'!
setUp

	super setUp.
	repository := self repositoryWithSample.
	FLFameObjectCluster defaultRepository: repository.
	analyzer := FLFameAnalyzer newDefault! !

!FLFameAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 8/24/2011 20:38'!
testClusteredObjectsOrder
	"The associations in clusteredObjects should be ordered ... (complete)"
	
	| yetNotVisitedClasses describedClasses |
	analyzer clusterizationOf: self sampleMooseModel entities.

	describedClasses := ((analyzer clusterization clusters 
		select: [ :aCluster | aCluster isKindOf: FLFameObjectCluster ]) 
		collect: [ :aCluster | aCluster theClass ]).
	
	self assert: describedClasses size >= 2 description: 'Case has not sense if classes are less than 2'.
	
	yetNotVisitedClasses := describedClasses copy.
	
	describedClasses
		do: [ :aClass | 
			yetNotVisitedClasses removeFirst.
			self assert: aClass notDependsOn: yetNotVisitedClasses]
 ! !

TestCase subclass: #FLFameMapperTest
	instanceVariableNames: 'mooseModel sampleDescription repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Tests'!

!FLFameMapperTest methodsFor: 'tests' stamp: 'MartinDias 11/25/2010 18:42'!
descriptionOfSampleIn: aRepository
		
	^ aRepository elements detect: [ :x | x class = FAMIXClass and: [ x name = #FLPair] ]! !

!FLFameMapperTest methodsFor: 'running' stamp: 'MartinDias 11/16/2010 23:53'!
repositoryWithSample
	
	^ (FMRepository with: MooseModel meta) 
			addAll: self sampleMooseModel entities;
			yourself.! !

!FLFameMapperTest methodsFor: 'tests' stamp: 'MartinDias 8/11/2011 14:54'!
sampleMapper
	^ FLFameMapper new! !

!FLFameMapperTest methodsFor: 'running' stamp: 'MartinDias 11/25/2010 18:41'!
sampleMooseModel

	^ MooseSqueakClassPackageImporterTask new
		model: MooseModel new;
		addClass: FLPair;
		run;
		model.
! !

!FLFameMapperTest methodsFor: 'running' stamp: 'MartinDias 8/12/2011 00:12'!
setUp

	super setUp.
	repository := self repositoryWithSample.
	FLFameObjectCluster defaultRepository: repository.
	sampleDescription := self descriptionOfSampleIn: repository.
! !

!FLFameMapperTest methodsFor: 'tests' stamp: 'MartinDias 5/30/2011 17:12'!
testAccepts

	self
		assert: (self sampleMapper accepts: sampleDescription)
		description: 'It should accept analyzing an object whose class is described in the repository'! !

!FLFameMapperTest methodsFor: 'tests' stamp: 'MartinDias 5/30/2011 17:12'!
testDoesNotAccept

	self 
		deny: (self sampleMapper accepts: 42) 
		description: 'It should not accept analyzing an object whose class is not described in the repository'! !

FLClusterization subclass: #FLFameClusterization
	instanceVariableNames: 'fameObjectsBucket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core'!

!FLFameClusterization methodsFor: 'private' stamp: 'MartinDias 8/12/2011 00:08'!
classDependenciesOf: aClass

	self flag: #todo. "access the repository in another way"

	^ ((((FLFameObjectCluster defaultRepository metamodel descriptionOf: aClass) allComplexAttributes)
		reject: [ :anAttribute | anAttribute isDerived ])
		collect: [ :anAttribute | anAttribute type implementingClass ]) asSet! !

!FLFameClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/24/2011 20:34'!
clusterBuckets

	^super clusterBuckets 
		add: self sortedFameClusters;
		yourself! !

!FLFameClusterization methodsFor: 'accessing' stamp: 'MartinDias 8/24/2011 20:21'!
fameObjectsBucket
	
	^ fameObjectsBucket! !

!FLFameClusterization methodsFor: 'initialize-release' stamp: 'MartinDias 8/24/2011 20:27'!
initialize

	super initialize.
	fameObjectsBucket := IdentitySet new.! !

!FLFameClusterization methodsFor: 'private' stamp: 'MartinDias 8/24/2011 20:24'!
lessDependedOf: clusters
	"Needed for sorting Fame clusters"

	^ clusters detect: [ :aCluster | 
		clusters allSatisfy: [ :anotherCluster | 
			aCluster == anotherCluster or: [ 
				((self classDependenciesOf: aCluster theClass) 
					anySatisfy: [ :aClass | anotherCluster theClass isKindOf: aClass ]) not ] ] ]! !

!FLFameClusterization methodsFor: 'private' stamp: 'MartinDias 8/24/2011 20:36'!
sortedFameClusters
	"Returns the clusters with Fame objects sorted: clusters of classes with less dependencies are at first."

	| sortedClusters yetNotSortedClusters |
	sortedClusters := OrderedCollection new.
	yetNotSortedClusters := fameObjectsBucket asOrderedCollection.

	[ yetNotSortedClusters isEmpty ] whileFalse: [ 
		| next |
		next := self lessDependedOf: yetNotSortedClusters..
		sortedClusters add: next.
		yetNotSortedClusters remove: next ].

	^ sortedClusters! !

!FLMaterializer methodsFor: '*FuelFameExtension' stamp: 'MartinDias 8/12/2011 00:12'!
materializeEntitiesDescribedWith: aMetamodel
	"Materializes Fame entities from the stream, and returns that result"

	| entities tower model |
	self flag: #todo. "Some of these steps are here just to imitate the MSE behaviour but could be not needed"
	tower := FMCompleteTower new.
	tower metamodel addAll: aMetamodel elements.

	FLFameObjectCluster defaultRepository: tower model.
	entities := self materialize; root.

	tower model addAll: entities.
	tower model updateCache.
	model := MooseModel new.
	tower model elements do: [ :e | model add: e ].
	model entityStorage forRuntime.
	^ model
! !

FLObjectCluster subclass: #FLFameObjectCluster
	instanceVariableNames: 'repository attributes classDescription'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core-Clusters'!

FLFameObjectCluster class
	instanceVariableNames: 'defaultRepository'!

!FLFameObjectCluster commentStamp: '<historical>' prior: 0!
I serialize and materialize objects using its Famix meta description. 

At the moment, the metamodel should be the same when serializing and materializing, or incongruences could appear.!

FLFameObjectCluster class
	instanceVariableNames: 'defaultRepository'!

!FLFameObjectCluster classSide methodsFor: 'accessing' stamp: 'MartinDias 8/12/2011 00:05'!
defaultRepository
	^ defaultRepository! !

!FLFameObjectCluster classSide methodsFor: 'accessing' stamp: 'MartinDias 8/12/2011 00:05'!
defaultRepository: anObject
	defaultRepository := anObject! !

!FLFameObjectCluster classSide methodsFor: 'instance creation' stamp: 'MartinDias 8/12/2011 00:14'!
for: aClass

	^ self basicNew
		theClass: aClass;
		initialize;
		yourself.! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 8/12/2011 01:59'!
attributes

	^attributes ifNil: [ attributes := (repository metamodel descriptionOf: theClass) allAttributes reject: [ :anAttribute | anAttribute isDerived ] ].! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 9/4/2011 03:52'!
classDescription

	^classDescription ifNil: [classDescription := repository metamodel descriptionOf: theClass].
! !

!FLFameObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/24/2011 20:21'!
clusterBucketIn: aClusterization

	^aClusterization fameObjectsBucket! !

!FLFameObjectCluster methodsFor: 'initialize-release' stamp: 'MartinDias 8/12/2011 01:54'!
initialize

	super initialize.
	repository := self class defaultRepository.
	self flag: #todo. "As a workaround, the class holds the repository to use. Instead, the instance should receive it in other way."! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/12/2011 23:18'!
materializeAttribute: anInstance of: anAttribute with: aMaterializer

	| valuesSize values |
	valuesSize := aMaterializer nextEncodedReference.

	self flag: #todo. "Encoding for skip ignored values (see #serializeIgnoredValueWith:)"
	(valuesSize = 0) 
		ifTrue: [ ^ self ].

	values := Array new: valuesSize.
	1 to: valuesSize do: [ :index | 
		values at: index put: aMaterializer nextEncodedReference ].	
		
	anAttribute 
		setOn: anInstance 
		values: values! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/4/2011 03:56'!
materializeInstanceWith: aMaterializer

	^self classDescription createInstance! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/4/2011 03:15'!
materializeReferencesStepWith: aMaterializer

	objects do: [ :anInstance |
		self attributes do: [ :anAttribute |
			self 
				materializeAttribute: anInstance 
				of: anAttribute 
				with: aMaterializer ] ]! !

!FLFameObjectCluster methodsFor: 'analyzing' stamp: 'MartinDias 8/12/2011 23:24'!
referencesOf: anObject do: aBlock

	self attributes 
		do: [ :anAttribute |
			| values |
			values := repository get: anAttribute element: anObject.
			(self shouldIgnore: anAttribute withAll: values)
				ifTrue: [ aBlock value: 0 ]
				ifFalse: [ 
					aBlock value: values size.
					values do: [ :aValue | aBlock value: aValue ] ] ]! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 11/16/2010 23:43'!
repository
	^ repository! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 11/16/2010 23:43'!
repository: anObject
	repository := anObject! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/24/2010 10:24'!
serializeAttribute: anAttribute values: values with: aSerializer

	(self shouldIgnore: anAttribute withAll: values)
		ifTrue: [ self serializeIgnoredValueWith: aSerializer ]
		ifFalse: [ self serializeValues: values with: aSerializer ]
! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/12/2011 23:26'!
serializeIgnoredValueWith: aSerializer

	self flag: #todo. "Encode in another way that its an ignored value."
	aSerializer encodeReferenceTo: 0! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/4/2011 03:55'!
serializeInstance: anObject with: aSerializer! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 9/4/2011 03:17'!
serializeReferencesStepWith: aSerializer

	objects do: [ :anInstance |
		self attributes do: [ :anAttribute |
			self 
				serializeAttribute: anAttribute 
				values: (repository get: anAttribute element: anInstance) 
				with: aSerializer ] ].! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 8/12/2011 23:17'!
serializeValues: values with: aSerializer

	aSerializer encodeReferenceTo: values size.
	values do: [ :aValue | aSerializer encodeReferenceTo: aValue ]! !

!FLFameObjectCluster methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/24/2010 10:22'!
shouldIgnore: attribute withAll: values
	"Copied from FMRepositoryVisitor>>ignoreProperty:withAll:"

	^ values isEmpty or: [
		attribute isDerived or: [
			attribute type == FM3MetaDescription boolean and: [
				values size == 1 and: [ 
					values first == false ]]]]! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 17:08'!
theClass

	^ theClass! !

!FLFameObjectCluster methodsFor: 'accessing' stamp: 'MartinDias 5/30/2011 17:08'!
theClass: anObject

	theClass := anObject! !

FLMapper subclass: #FLFameMapper
	instanceVariableNames: 'clusters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core-Mappers'!

!FLFameMapper commentStamp: '<historical>' prior: 0!
I am an ObjectAnalyzer for objects whose class is described in Famix.

I work together with FLFameAnalyzer.!

!FLFameMapper methodsFor: 'accessing' stamp: 'MartinDias 8/12/2011 00:09'!
accepts: anObject

	FLFameObjectCluster defaultRepository metamodel 
		descriptionOf: anObject class 
		ifAbsent: [ ^false ].
	^ true! !

!FLFameMapper methodsFor: 'evaluating' stamp: 'MartinDias 8/12/2011 00:11'!
clusterFor: anObject

	^clusters
		at: anObject class
		ifAbsentPut: [ FLFameObjectCluster for: anObject class ]! !

!FLFameMapper methodsFor: 'initialize-release' stamp: 'MartinDias 8/12/2011 00:10'!
initialize

	super initialize.
	clusters := IdentityDictionary new.! !
